VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "TextPos"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Description = "Searches through text file and returns the byte position of the first occurance."
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

'Object Level Variables
Private Type Search
    WindowSize As Long      'Number of Bytes to read from disk in each pass
    StartPosition As Long   'Starting byte position for search
    String As String        'Text string to search for
    FileName As String      'Fully qualified path of file to search
    AdjacentDigits As Long  'Specify the number of digits to return from the left and right of search string
    BCDMode As Integer      'Specify if search file is in Binary Coded Decimal Format
End Type
    
Private Type Result
    TextPosition As Long    'The starting byte position at which the search string was found
    SearchTime As Single    'Search processing time in seconds
    FileSize As Long        'The total number of bytes in search file
    ErrorCode As Long       'Error code (if any) reported
    LeftDigits As String    'Digits to Left of Search String
    RightDigits As String   'Digits to Right of Search String
End Type

Dim Result As Result
Dim Search As Search

'Error Constants
Private Enum ERROR
    BASE = vbObjectError
    UNSPECIFIED = vbObjectError + 1
    SEARCH_FILE_NOT_FOUND = vbObjectError + 2
    ZERO_FILE_LENGTH = vbObjectError + 3
    INVALID_WINDOW_SIZE = vbObjectError + 4
    SEARCH_STRING_NOT_SPECIFIED = vbObjectError + 5
    START_POSITION_AFTER_EOF = vbObjectError + 6
End Enum
    
Sub InitializeResult()

    Result.ErrorCode = 0
    Result.FileSize = 0
    Result.LeftDigits = Empty
    Result.RightDigits = Empty
    Result.SearchTime = 0
    Result.TextPosition = 0
    
End Sub


Public Property Get SearchWindowSize() As Long

    SearchWindowSize = Search.WindowSize
    
End Property

Public Property Let SearchWindowSize(ByVal vNewValue As Long)

    Search.WindowSize = vNewValue
    
End Property

Public Property Get SearchString() As String

    SearchString = Search.String
    
End Property

Public Property Let SearchString(ByVal vNewValue As String)

    Search.String = vNewValue
    
End Property

Public Property Get SearchFileName() As String

    SearchFileName = Search.FileName
    
End Property

Public Property Let SearchFileName(ByVal vNewValue As String)

    Search.FileName = vNewValue
    
End Property

Public Property Get ResultSearchTime() As Single
        
    ResultSearchTime = Result.SearchTime

End Property

Public Property Get ResultTextPosition() As Long

    ResultTextPosition = Result.TextPosition
    
End Property

Public Property Get ResultErrorCode() As Long

    ResultErrorCode = Result.ErrorCode
    
End Property

Private Function FileExist(FileName As String) As Boolean
'---------------------------------------------------------------------------
'{FileExist} checks for [FileName] and returns True if exists of False if
'   the file doesn't exist or is in use by another process.
'
'---------------------------------------------------------------------------

On Error GoTo PathExistError

    Open FileName For Input As #1
    Close #1
    FileExist = True
    Exit Function

PathExistError:
    FileExist = False
    Exit Function
    
End Function

Public Property Get SearchStartPosition() As Long

    SearchStartPosition = Search.StartPosition
    
End Property

Public Property Let SearchStartPosition(ByVal vNewValue As Long)

        Search.StartPosition = vNewValue
        
End Property

Private Function ValidateParms(ByVal FileSize As Long) As Long
'---------------------------------------------------------------------------
'{ValidateParms} validates object input and operating parameters.  Required
'   input parameters not specified by the called object instance are set
'   with default values.  Function returns 0 if all operating parameters
'   are valid or returns
'
'---------------------------------------------------------------------------

Const DEFAULT_SEARCH_WINDOW_SIZE = 131072
Const DEFAULT_SEARCH_START_POSITION = 1

    ValidateParms = False 'Set default result for function
    
    'Validate Search.WindowSize
    If Search.WindowSize <= 0 Then
        ValidateParms = ERROR.INVALID_WINDOW_SIZE
        Exit Function
    End If
    
    'Validate Search.FileName > 0 bytes
    If FileSize = 0 Then
        ValidateParms = ERROR.ZERO_FILE_LENGTH
        Exit Function
    End If
    
    'Validate and set Search.WindowSize
    Select Case Search.WindowSize
        Case 0
           If Result.FileSize > DEFAULT_SEARCH_WINDOW_SIZE Then
                Search.WindowSize = DEFAULT_SEARCH_WINDOW_SIZE
              Else
                Search.WindowSize = FileSize
            End If
        Case Is > FileSize
           Search.WindowSize = FileSize
    End Select
    
    'Validate Search.StartPosition
    Select Case Search.StartPosition
        Case Is > FileSize - Len(Search.String)
           ValidateParms = ERROR.START_POSITION_AFTER_EOF
            Exit Function
        Case Is < 1
            Search.StartPosition = DEFAULT_SEARCH_START_POSITION
    End Select
    
    'Validate Search.AdjacentDigits
    Select Case Search.AdjacentDigits
        Case Is > 255
            Search.AdjacentDigits = 255
        Case Is < 0
            Search.AdjacentDigits = 0
    End Select
        
    'Validate Search.String
    'If Search.String = "" Then
    '    Result.ErrorCode = ERROR.SEARCH_STRING_NOT_SPECIFIED
    '    Exit Function
    'End If
    
End Function

Public Property Get ResultFileSize() As Variant

    ResultFileSize = Result.FileSize
    
End Property


Public Function TextPos() As Boolean

On Error GoTo ErrorHandle

    'Search Engine Local Variables
    Dim SearchString As String      'Local copy of Search.String
    Dim BlockSize As Long           'Local copy of Search.WindowSize
    Dim StartPosition As Long       'Local copy of Search.StartPosition
    Dim BCDMode As Integer          'Local copy of Search.BCDMode
    Dim FileSize As Double          'The size of the file to be read
    Dim BufferBlock As String       'Space filled buffer string for file read operations
    Dim InputBlock As String        'Input block from file
    Dim BoundryBlock As String      'The end (StringLength - 1) of the previous InputBlock to catch catch strings lying on top of block boundaries
    Dim ReadBytePosition As Long    'The number of bytes processed
    Dim StringLength As Integer     'The length of Search.String
    Dim StartTime As Single         'Process start time
    Dim AdjacentDigits As Integer   'The number of digits adjacent to SearchString to retrieve
    Dim AdjacentDigitsStr As String 'The digits adjacent to search string
    Dim DigitLeftBounds As Long     'The Left Boundry of the Adjacent Digits to retrieve
    Dim DigitRightBounds As Long    'The Right Boundry of the Adjacent Digits to retrieve
    Dim retValidateParms As Long    'The result returned from ValidateParms
    
        Call InitializeResult
        
        StartTime = Timer           'Set process start time
        TextPos = False             'Set default result for function
        
        'Validate file exists
        If Not FileExist(Search.FileName) Then
            Result.ErrorCode = ERROR.SEARCH_FILE_NOT_FOUND
            Exit Function
        End If
        
        Open Search.FileName For Binary Access Read As #1
        FileSize = LOF(1)    'Get the total number of bytes in the file
        
        retValidateParms = ValidateParms(FileSize) 'Validate user parameters
            If retValidateParms <> 0 Then
            Err.Raise retValidateParms
        End If
    
        'Initialize Local Copies User Variables
        SearchString = Search.String
        StartPosition = Search.StartPosition
        AdjacentDigits = Search.AdjacentDigits
        BlockSize = Search.WindowSize
        BCDMode = Search.BCDMode
        
        'Initialize Process Variables
        StringLength = Len(SearchString)
        BoundryBlock = ""                           'First through boundry block
        ReadBytePosition = Search.StartPosition     'First byte to search
    
        '-------------------
        'Search Engine Begin
        '-------------------
        
        If SearchString = "" And AdjacentDigits > 1 Then 'Return Digits from StartPosition
            InputBlock = Space(AdjacentDigits)
            Get #1, ReadBytePosition, InputBlock    'Fill InputBlock with data from file
            Close #1
            TextPos = True
            Result.TextPosition = StartPosition
            Result.RightDigits = strTrimWhiteSpace(InputBlock)
          Else
            BufferBlock = Space(BlockSize)
            Do 'Execute Search
                InputBlock = BufferBlock
                Get #1, ReadBytePosition, InputBlock 'Fill InputBlock with data from file
                InputBlock = BoundryBlock + InputBlock
                Select Case InStrB(1, InputBlock, SearchString, vbBinaryCompare)
                    Case 0      'Match not found
                        BoundryBlock = Right(InputBlock, StringLength - 1) 'Prep for next block from file
                    Case Else   'Match found
                        Result.TextPosition = (ReadBytePosition - 1) + InStr(1, InputBlock, SearchString) - Len(BoundryBlock)
                        If AdjacentDigits > 0 Then 'Get Adjacent Digits if requested
                            If (Result.TextPosition - AdjacentDigits) < 1 Then
                                DigitLeftBounds = 1
                              Else
                                DigitLeftBounds = Result.TextPosition - AdjacentDigits
                            End If
                            If (Result.TextPosition + AdjacentDigits) > FileSize Then
                                DigitRightBounds = FileSize
                              Else
                                DigitRightBounds = Result.TextPosition + AdjacentDigits + StringLength - 1
                            End If
                            AdjacentDigitsStr = Space((DigitRightBounds - DigitLeftBounds) + 1)
                            Get #1, (DigitLeftBounds), AdjacentDigitsStr
                            Result.LeftDigits = strTrimWhiteSpace(Left(AdjacentDigitsStr, Result.TextPosition - DigitLeftBounds))
                            Result.RightDigits = strTrimWhiteSpace(Right(AdjacentDigitsStr, DigitRightBounds - Result.TextPosition - StringLength + 1))
                        End If
                        Close #1
                        Exit Do
                End Select
                ReadBytePosition = ReadBytePosition + BlockSize
            Loop Until EOF(1)
        End If
        
        Close #1
        
        '-------------------
        'Search Engine End
        '-------------------
        
        'Return Result
        Result.SearchTime = Abs(Timer - StartTime)
        Result.FileSize = FileSize
        
        TextPos = True 'Search executed normally
        Exit Function
    
    
ErrorHandle:
        Result.ErrorCode = Err.Number
        Close #1
        TextPos = False
        
End Function

Public Property Get AdjacentDigits() As Integer

    AdjacentDigits = Search.AdjacentDigits
    
End Property

Public Property Let AdjacentDigits(ByVal vNewValue As Integer)

    Search.AdjacentDigits = vNewValue
    
End Property

Public Property Get ResultLeftDigits() As String

    ResultLeftDigits = Result.LeftDigits

End Property

Public Property Get ResultRightDigits() As Variant

    ResultRightDigits = Result.RightDigits
    
End Property

Public Property Let BCDMode(ByVal vNewValue As Integer)

    Search.BCDMode = vNewValue
    
End Property

