Attribute VB_Name = "UTIL"
Option Explicit
'Utility module updated 4/17/2009 - JamesT

Declare Function GetWindowsDirectory Lib "kernel32" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long



Public Function strStripCommas(ByVal strIN_Text) As String
'---------------------------------------------------------------------------
'strStripCommas() removes any commas from the input text
'
'---------------------------------------------------------------------------

    Const strREGEXP_PATTERN = ","
        
    strStripCommas = strFilterText(strIN_Text, strREGEXP_PATTERN, Empty)
    
End Function




Public Function strTrimWhiteSpace(ByVal strIN_Text) As String
'---------------------------------------------------------------------------
'strTrimWhiteSpace() removes any NULL, tab, CRLF or space characters from
'   the beginning or end of the input text
'
'---------------------------------------------------------------------------

    Const strREGEXP_PATTERN = "^[\x00\x09\x0a\x0d\x20]+|[\x00\x09\x0a\x0d\x20]+$"
        
    strTrimWhiteSpace = strFilterText(strIN_Text, strREGEXP_PATTERN, Empty)
    
End Function
Public Function strFilterText(ByVal strIN_Text, ByVal strIN_RegExpPattern, ByVal strIN_ReplacementText) As String
'---------------------------------------------------------------------------
'strFilterText filters text from a string based on the provided regular
'   expression pattern
'
'---------------------------------------------------------------------------

    Dim objRegExp As RegExp
        
    If strIN_Text = Empty Or IsNull(strIN_Text) Then
        strFilterText = Empty
        Exit Function
    End If
        
    Set objRegExp = New RegExp
        
    With objRegExp
        .IgnoreCase = True
        .Global = True
        .Pattern = strIN_RegExpPattern
    End With
    
    strFilterText = objRegExp.Replace(strIN_Text, strIN_ReplacementText)
    
    Set objRegExp = Nothing
        
End Function
Function ParseEndSpace(ByVal StringToParse As Variant) As String
'---------------------------------------------------------------------------
'Removes all spaces from the end of StringToParse and
'   returns a string
'
'---------------------------------------------------------------------------

Dim ParseStringPosition As Integer

    StringToParse = CStr(StringToParse)
    
    For ParseStringPosition = Len(StringToParse) To 1 Step -1
        If Mid(StringToParse, ParseStringPosition, 1) <> Chr(32) Then
            ParseEndSpace = Left(StringToParse, ParseStringPosition)
            Exit Function
        End If
    Next ParseStringPosition

End Function

Function SpaceParse(ByVal StringToParse As Variant) As String
'---------------------------------------------------------------------------
'Returns a string, regardless of the data type passed, eliminating any
'   spaces.
'
'---------------------------------------------------------------------------

Dim CharUnderTest As String
Dim BuildString As String
Dim StringPosition As Integer

    StringToParse = CStr(StringToParse)
    
    Do Until StringPosition = Len(StringToParse)
        StringPosition = StringPosition + 1
        CharUnderTest = Mid(StringToParse, StringPosition, 1)
        If CharUnderTest <> Chr(32) Then BuildString = BuildString & CharUnderTest
    Loop

    SpaceParse = BuildString

End Function



Public Function PathExist(ByVal Path As String) As Boolean
'---------------------------------------------------------------------------
'{PathExist} checks for [Path] and returns True if exists or False if the
'   path doesn't exist.  Returns FALSE if filename is included in the path.
'
'---------------------------------------------------------------------------

On Error GoTo PathExistError

    ChDir Path
    PathExist = True
    Exit Function

PathExistError:
    PathExist = False
    Exit Function

End Function

Public Function ItemParse(ByVal StringUnderTest As String, ByVal ItemPosition As Integer, Optional ByVal Delimiter As String = ",") As String
'---------------------------------------------------------------------------
'ItemParse will locate an item within a delimited string by passing the
'   position (ItemPosition) and delimiter (Delimiter; optional, defaults
'   to ",").
'
'---------------------------------------------------------------------------

Dim TagCount As Integer, LastTag As Integer
Dim ItemTag(255) As Integer

    If ItemPosition <= 0 Then
        ItemParse = ""
        Exit Function
    End If

    If InStr(StringUnderTest, Delimiter) = 0 And ItemPosition > 1 Then
        ItemParse = ""
        Exit Function
    End If
    
    If InStr(StringUnderTest, Delimiter) = 0 And ItemPosition = 1 Then
        ItemParse = StringUnderTest
        Exit Function
    End If

    Do
        TagCount = TagCount + 1
        ItemTag(TagCount) = InStr(LastTag + 1, StringUnderTest, Delimiter)
        LastTag = ItemTag(TagCount)
    Loop Until LastTag = 0 Or TagCount = ItemPosition + 1

    ItemTag(TagCount) = Len(StringUnderTest) + 1
    
    If ItemPosition > TagCount Then
        ItemParse = ""
        Exit Function
    End If
    
    ItemParse = Mid(StringUnderTest, ItemTag(ItemPosition - 1) + 1, ItemTag(ItemPosition) - ItemTag(ItemPosition - 1) - 1)

End Function

Public Function ItemCount(ByVal StringUnderTest As String, Optional ByVal Delimiter As String) As Integer
'---------------------------------------------------------------------------
'ItemCount will return the number of items in a delimited string by
'   passing the delimiter(Deimiter; optional, defaults to ",")
'
'---------------------------------------------------------------------------

Dim LastTag As Integer, TagCount As Integer
Dim ItemTag(255) As Integer

    If StringUnderTest = "" Then
        ItemCount = 0
        Exit Function
    End If
    
    If Delimiter = "" Then Delimiter = ","
    
    If InStr(StringUnderTest, Delimiter) = 0 Then
        ItemCount = 1
        Exit Function
    End If
    
    TagCount = 1
    Do
        ItemTag(TagCount) = InStr(LastTag + 1, StringUnderTest, Delimiter)
        LastTag = ItemTag(TagCount)
        TagCount = TagCount + 1
    Loop Until LastTag = 0

    LastTag = Len(StringUnderTest)
    ItemTag(TagCount) = LastTag
    ItemCount = TagCount - 1

End Function


Public Function AsciiFilter(ByVal Ascii As Integer, ByVal Mode As Integer) As Integer
'---------------------------------------------------------------------------
'Filter ASCII Codes based on MODE.  This function will allways pass
'   the Backspace and Delete codes.
'
'Mode = 1 - Alpha
'       2 - Numeric
'       3 - Alpha-Numeric
'       4 - Valid Path Name Characters
'
'--------------------------------------------------------------------------
    
    Select Case Mode
        Case 1
            Select Case Ascii
                Case 8, 65 To 90, 97 To 122
                    AsciiFilter = Ascii
            End Select
        Case 2
            Select Case Ascii
                Case 8, 48 To 57
                    AsciiFilter = Ascii
            End Select
        Case 3
            Select Case Ascii
                Case 8, 48 To 57, 65 To 90, 97 To 122
                    AsciiFilter = Ascii
            End Select
        Case 4
            Select Case Ascii
                Case 34, 42, 47, 60, 62, 63, 124
                    AsciiFilter = 0
                Case Else
                    AsciiFilter = Ascii
            End Select
    End Select
    
End Function

Public Function FileExist(ByVal FileName As String) As Boolean
'---------------------------------------------------------------------------
'{FileExist} checks for [FileName] and returns True if exists of False if
'   the file doesn't exist or is in use by another process.
'
'---------------------------------------------------------------------------

On Error GoTo PathExistError

    Open FileName For Input As #1
    Close #1
    FileExist = True
    Exit Function

PathExistError:
    FileExist = False
    Exit Function

End Function

Public Function WinPath() As String
'---------------------------------------------------------------------------
'{WinPath} returns the path of the Operating System
'
'---------------------------------------------------------------------------

Dim Buffer As String

    Buffer = String(145, 0)
    WinPath = Left(Buffer, GetWindowsDirectory(Buffer, Len(Buffer)))

End Function

Public Static Function AppPath()
'---------------------------------------------------------------------------
'This function returns the full path that the application
'   was executed from, including the closing back-slash
'   on root-paths.
'
'---------------------------------------------------------------------------

    If Right(App.Path, 1) = "\" Then
        AppPath = App.Path
      Else
        AppPath = App.Path & "\"
    End If
        
End Function

Public Function PathExistFull(ByVal Path As String) As Boolean
'---------------------------------------------------------------------------
'{PathExistFull} checks for [Path] and returns True if exists or False if the
'   path doesn't exist.  Returns TRUE if a valid filename is included in the
'   path.
'
'---------------------------------------------------------------------------
On Error GoTo ErrorHandle
Dim PathLength As Integer, FileName As String, FileNameLength As Integer, Char As Integer
Const DELIM = "\"

    PathLength = Len(Path)
    FileName = ItemParse(Path, ItemCount(Path, DELIM), DELIM)
    FileNameLength = Len(FileName)
    Path = Left(Path, PathLength - FileNameLength - 1)
    
    Select Case Dir(Path, 55)
        Case "", "."
            PathExistFull = False
            Exit Function
        Case Else 'Ensure filename is valid
            If ValidFilePathName(FileName) = False Then
                PathExistFull = False
                Exit Function
            End If
    End Select
    
    PathExistFull = True
    Exit Function
    
ErrorHandle:
    PathExistFull = False
    
End Function

Public Function ValidFilePathName(ByVal FilePath As String) As Boolean
'---------------------------------------------------------------------------
'{ValidFilePathName} checks FilePath for invalid File/Path characters and
'   returns True if all are valid or False invalid characters exist
'   (/ * ? " < > |)
'
'---------------------------------------------------------------------------
Dim Char As Integer, FilePathLength As Integer

    FilePathLength = Len(FilePath)
            
    For Char = 1 To FilePathLength 'Check for invalid filename characters
        If AsciiFilter(Asc(Mid(FilePath, Char, 1)), 4) = 0 Then
            ValidFilePathName = False
            Exit Function
        End If
    Next
    
    ValidFilePathName = True

End Function

Public Function RoundUp(ByVal dblInputValue As Double) As Double

    Dim lngDecimalPosition As Long
 
    lngDecimalPosition = InStr(1, CStr(dblInputValue), ".", vbTextCompare)
    
    If lngDecimalPosition > 0 Then
        RoundUp = CDbl(Left(CStr(dblInputValue), lngDecimalPosition)) + 1
      Else
        RoundUp = dblInputValue
    End If

End Function
Public Function RoundDown(ByVal dblInputValue As Double) As Double

    Dim lngDecimalPosition As Long
 
    lngDecimalPosition = InStr(1, CStr(dblInputValue), ".", vbTextCompare)

    If lngDecimalPosition > 0 Then
        RoundDown = CDbl(Left(CStr(dblInputValue), lngDecimalPosition))
      Else
        RoundDown = dblInputValue
    End If

End Function
