Option Explicit

'---------------------------------------------------------------------------
'The VBScript is a demonstration of a technique to automate the use of the
'	TextPOS.dll library to locate the positions or occurances of a
'	series of strings within a text file (e.g. the digits of PI).
'
'	The search arguments are provided in an input file, which is stored
'	in the constant strSEARCH_TEXT_FILE, delimited by CRLF.  The script
'	will enumerate through the file, locating each occurance of the
'	string, and if found, output the byte position of the first digit.
'	The script will then attempt to locate the next occurance, starting
'	at the previous location + 1.
'
'	The output is comma delimited with the following fields:
'
'	* CurrentTime - The local time that the search was initiated
'	* SearchString - The numeric string being searched for
'	* TextPosition - The starting digit position of search string, if
'			 found.  0 is returned if not found
'	* SearchTime - The time taken to complete the search
'	* ErrorCode - 0 if the function returned normally
'
'
'	***The script assumes that the TextPOS.dll class has been registered
'	on the target workstation.***
'
'---------------------------------------------------------------------------

Dim objTextPos, blnTextPosResCode, blnFoundFlag
Dim resSearchString, resTextPosition, resSearchTime, resSearchStartPosition, resErrorCode
Dim resLeftDigits, resRightDigits, resFileSize
Dim strSearchTextFile, lngArrayIndex
Dim lngSearchStartPosition
Dim strStartTime

Dim objFileSystem, objSearchTextFile
Dim SearchArray

Const strSEARCH_TEXT_FILE = ".\searchdigits.txt"		'The file containing the search digits, CRLF delimited
Const strSEARCH_TARGET_FILE = "f:\Webfiles\pi\pi_2b.txt"	'The file containing the constant string to be searched through (i.e. pi, e, phi, etc)

strStartTime = Now()

    Set objFileSystem = CreateObject("Scripting.FileSystemObject")
    Set objSearchTextFile = objFileSystem.OpenTextFile(strSEARCH_TEXT_FILE , 1)

    strSearchTextFile = objSearchTextFile.ReadAll
    SearchArray = Split(strSearchTextFile, vbCRLF)

     Call WriteLogFile("CurrentTime,SearchString,TextPosition,SearchTime,ErrorCode")

     For lngArrayIndex = LBound(SearchArray) To UBound(SearchArray)
        lngSearchStartPosition = 1
	blnFoundFlag = False
        
        Do Until lngSearchStartPosition = 0
            Set objTextPos = CreateObject("TextPositionSearch.TextPos")
           
                With objTextPos
                    .SearchString = SearchArray(lngArrayIndex)
                    .SearchFileName = strSEARCH_TARGET_FILE
                    .SearchWindowSize = 65536
                    .SearchStartPosition = lngSearchStartPosition
                    .AdjacentDigits = 0
                    blnTextPosResCode = .TextPos
                End With
                
                With objTextPos
                    resSearchString = .SearchString
                    resTextPosition = .ResultTextPosition
                    resLeftDigits = .ResultLeftDigits
                    resRightDigits = .ResultRightDigits
                    resSearchTime = .ResultSearchTime
                    resFileSize = .ResultFileSize
                    resSearchStartPosition = .SearchStartPosition
                    resErrorCode = .ResultErrorCode
                End With
                
            Set objTextPos = Nothing
            
	    If CLng(resTextPosition) = 0 Then
                'String not found
		lngSearchStartPosition = 0
		blnFoundFlag = Not(blnFoundFlag)
              Else
                'String Found - increment the counter
		lngSearchStartPosition =  resTextPosition + 1
		blnFoundFlag = True
            End If

            If blnFoundFlag Then Call WriteLogFile(Now() & "," & resSearchString & "," & resTextPosition & "," & resSearchTime & "," & resErrorCode)

        Loop
        
    Next
	
    Set objSearchTextFile = Nothing
    Set objFileSystem = Nothing
    
    Call WriteLogFile("Total Processing Time: " & FormatDateTime(Now() - strStartTime))
   
'---------------------------------------------------------------------------

Sub WriteLogFile(strLogEntry)
	
	Const strLOG_FILE_EXTENSION = "csv"

	Dim objFileSystem, objLogFile, objLogFile_TextStream, strLogFileName
	
	Set objFileSystem = CreateObject("Scripting.FileSystemObject")
		
	strLogFileName = Mid(Wscript.ScriptName,1,Len(Wscript.ScriptName)-3) & strLOG_FILE_EXTENSION

	If Not objFileSystem.FileExists (strLogFileName) then
		Set objLogFile = objFileSystem.CreateTextFile(strLogFileName)
		objLogFile.Close
	End If

	Set objLogFile = objFileSystem.GetFile(strLogFileName)
	Set objLogFile_TextStream = objLogFile.OpenAsTextStream(8, 0)
	
	objLogFile_TextStream.WriteLine(strLogEntry)	
	objLogFile_TextStream.Close

	Set objLogFile_TextStream = Nothing
	Set objLogFile = Nothing
	Set objFileSystem = Nothing

End Sub